var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React from 'react';
import { omit } from 'lodash';
import { Context } from './context';
import { calculateRescale } from './rescale-utils';
import withDimensions from './with-dimensions';
import { execChildrenFunctions } from './react-utils';
import DebugGrid from './DebugGrid';

export default withDimensions(class Viz extends React.Component {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.rescale = calculateRescale([0, 0], [1, 1]), _temp;
  }

  componentWillMount() {
    this.componentWillUpdate(this.props);
  }

  componentWillUpdate(nextProps) {
    const { margin, dimensions, flipY } = nextProps;
    const { width, height } = dimensions;
    this.rescale = calculateRescale([0, 0], [width, height], margin, { flipY });
  }

  render() {
    const _props = this.props,
          { children, mouse, debug } = _props,
          otherProps = _objectWithoutProperties(_props, ['children', 'mouse', 'debug']);
    const { rescale } = this;
    const svgProps = omit(otherProps, ['dimensions', 'margin', 'flipY']);
    const listeners = mouse ? buildListeners(otherProps, mouse, rescale) : {};

    return React.createElement(
      Context,
      { rescale: rescale },
      React.createElement(
        'svg',
        _extends({ width: '100%', height: '100%' }, svgProps, listeners),
        debug && React.createElement(DebugGrid, { rescale: rescale, color: typeof debug === 'string' ? debug : 'steelblue' }),
        execChildrenFunctions(children, [rescale])
      )
    );
  }
});

function buildListeners(props, mouse, rescale) {
  const listeners = ['onClick', 'onMouseDown', 'onMouseUp', 'onMouseMove'];
  return listeners.reduce((obj, key) => {
    const originalListener = props[key];
    obj[key] = event => {
      mouse.fromEvent(event, rescale);
      if (originalListener) originalListener(mouse);
    };
    return obj;
  }, {});
}